//
//  ideball_v02.c
//  ideball_v02
//
//  Created by lixouda on 6/29/11.
//  Copyright RWTH AACHEN 2011. All rights reserved.
//



#include <avr/io.h>
#include <util/delay.h>
#include "rfid.h"
#include "uart.h"
#include "accelerometer.h"
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
//#include <avr/sleep.h>
#include <inttypes.h>
#include <avr/wdt.h>
#include "ringbuff.h"



//void shakeCheck();

volatile char tag[11]; 
char tagsArray[3][11]= {"01065E928C","01065E82E5","1100339F83"};
extern volatile uint8_t bytesread;
extern volatile uint8_t tagReceived;
extern volatile char tag[11]; 
extern volatile int shaked;
extern volatile int z_shake;
extern volatile int y_shake;
extern volatile int x_shake;

extern uint8_t val;

RingBuff_t receiveBuff;

#define Reset_AVR() wdt_enable(WDTO_4S); while(0){}
#define START 0
#define NAVIGATORCONNECT 1
#define WAITRECORD 2
#define RECORD 3
#define RECORDSAFE 4
#define END 5


#define LEDSPORT PORTB
#define LEDSDDRPORT DDRB
#define RED PINB0
#define BLUE PINB1
#define GREEN PINB2
#define COLORS 6
#define LEDSPORTOFFSET 3


#define BUZZPORT PORTB
#define BUZZDDRPORT DDRB
#define BUZZPIN PINB3


ISR(USART1_RX_vect) {
	
	Buffer_StoreElement(&receiveBuff, UDR1);
}


void ledPortInit(void);
void buzz(void);
void readString(uint8_t incomingstring[], int len);
uint8_t compareStringToInput( uint8_t* string, int length);
uint8_t compareStrings(uint8_t* string1, uint8_t* string2, int length);
void copyString(uint8_t* fromString, uint8_t* toString, int length);


int main (void) {
	bytesread = 0;
    
	uint8_t state = START;
	uint8_t btconnectString[10] = "BTCONNECT";
    uint8_t oldtag[10] = "0000000000";
	uint8_t count;
    uint8_t tagReply = '*';
    uint8_t connectReq = 'x';
    uint8_t connchar= 'm';
    uint8_t disconnchar = 'q';
	cli();	
	USART0Init(UBRR0VAL);
	USART1Init(9);
	acceler_ioinit();
	ledPortInit();
    BUZZDDRPORT |= (1 << BUZZPIN);
	Buffer_Initialize(&receiveBuff);
	
	/*	
	 // Clear WDRF in MCUSR
	 MCUSR &= ~(1<<WDRF);
	 // Write logical one to WDCE and WDE 
	 // Keep old prescaler setting to prevent unintentional time-out
	 WDTCSR |= (1<<WDCE) | (1<<WDE); 
	 // Turn off WDT
	 WDTCSR = 0x00;
	 */
	
	sei();
	
	//uart1_write('&');
	//	Reset_AVR()
	for (;;) 
	{
		
		switch (state) {
			case START:	
				LEDSPORT |= (1 << RED);
				if (compareStringToInput(&connchar,1)) 
				{
					state++;
                   //receiveBuff.Elements = 0;
				}
				break;
				
			case NAVIGATORCONNECT:
				LEDSPORT &= ~(1 << RED) & ~(1 << GREEN);
				LEDSPORT |= ( 1 << BLUE);
                if (shakeCheck())
                {
                    uart1_write(connectReq);
                    state++;
                }
                break;
            case WAITRECORD:
                LEDSPORT |= ( 1 << RED);
                if (compareStringToInput(&connectReq,1))
                {
                    state++;
                    //receiveBuff.Elements = 0;

                }
				break;
				
			case RECORD:
                x_shake = 0;
                y_shake = 0;
                z_shake = 0;

                
				LEDSPORT &= ~(1 << RED) & ~(1 << BLUE);
				LEDSPORT |= ( 1 << GREEN);
             	if (rfidCheck() == 1)
                {
                    //compare this tag with the old tag
                    if (compareStrings(oldtag, tag, 10) == 0)
                    {
                        copyString(tag, oldtag, 10);
                        uart1_write(tagReply);
                        for (count = 0; count < 10; count++)
                        {
                            uart1_write(tag[count]);
                        }
                    }
                    tagReceived = 0;
                    state++;
                    
                }
                break;
            case RECORDSAFE:

                LEDSPORT &= ~(1 << RED) & ~(1 << BLUE);
				LEDSPORT |= ( 1 << GREEN);
             	if (rfidCheck() == 1)
                {
                    //compare this tag with the old tag
                    if (compareStrings(oldtag, tag, 10) == 0)
                    {
                        copyString(tag, oldtag, 10);
                        uart1_write(tagReply);
                        for (count = 0; count < 10; count++)
                        {
                            uart1_write(tag[count]);
                        }
                    }
                    tagReceived = 0;
                }
                if (compareStringToInput(&tagReply,1))
                {
                    buzz();
                }
                if (shakeCheck())
                {
                    //inform that we want to disconnect
                    uart1_write(disconnchar);
                    state = END;
                }
				break;
			case END:
				LEDSPORT &= ~(1 << BLUE);
				LEDSPORT |= ( 1 << GREEN) | ( 1 << RED);
				break;
				
			default:
				//uart1_write('&');
				break;
		}
	}	
    return 0;
}

void buzz(void)
{
    BUZZPORT |= ( 1 << BUZZPIN);
    _delay_ms(4);
    BUZZPORT &= ~( 1 << BUZZPIN);
}

void ledPortInit(void)
{
	LEDSDDRPORT |= (1 << RED) | ( 1 << BLUE) | ( 1 << GREEN);
	LEDSPORT &= ~(1 << RED) & ~( 1 << BLUE) & ~(1 << GREEN);
	
}


void readString(uint8_t incomingstring[], int len)
{
	int i;
	for (i = 0; i < len; i++)
	{	
		incomingstring[i] =  Buffer_GetElement(&receiveBuff);   // = uart_waitandread(USART1);
		
	}
	
}

void copyString(uint8_t* fromString, uint8_t* toString, int length)
{
    int i = 0;
    for (i = 0; i < length; i++)
    {
        toString[i] = fromString[i];
    }
}
uint8_t compareStrings(uint8_t* string1, uint8_t* string2, int length)
{
    int i = 0;
    for (i = 0; i < length; i++)
    {
        if (string1[i] != string2[i])
            return 0;
    }
    return 1;
    
}

uint8_t compareStringToInput(uint8_t* string, int length)
{
	int i = 0;
	_delay_ms(1);
	for ( i = 0; i < length; i++) 
	{
		if (receiveBuff.Elements) 
		{
			if (Buffer_GetElement(&receiveBuff)!= string[i])
			{
				return 0; //return 0 if not equivalent
			}
		}
		else {
			return 0;
		}
				
	}
	return 1; //return 1 if the string is equal to the input string
}

